// Class which generate a random noise signal at a given sampling rate. Then
// allows the user to take samples from the signal using interpolation if
// necessary.  It allows the generation of three separate noise senquences
// by starting at random locations within the original noise sequence.
// This code is adapted from Greg Welch at UNC.

#ifndef _GAUSS
#define _GAUSS

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

class GaussWhiteNoise
{
  // Provides a (zero mean) normally distributed white noise
  // source with a particular standard deviation (variance)

  public:

    GaussWhiteNoise(unsigned int seed);
    GaussWhiteNoise(char* noise_file, unsigned int seed, int verbose = 1);
    ~GaussWhiteNoise();

    void generateNoise(double srate, double duration, double variance, int cap = 0);
    void resetCounters();
    void deleteNoise();
    void randomizeCounters(unsigned int seed);    

    double getVariance() { return _var; }
    
    double sample();
    double sample(double dt);
    double sample(double dt, int i);
      

  private:

    unsigned short int _xsubi[3]; // random number state
    double _var; // Save the variance
    double _sampleRate;
    double _duration;
    double _last_fds;
    double _last_fds2;
    double _last_fds3;

    double* _noise;
    // want to be able to generate one large sequence and use it to
    // to add noise to three distinct components of a pose
    double* _noiseFirst;
    double* _noiseCurrent;
    double* _noiseLast;

    double* _noiseFirst2;
    double* _noiseCurrent2;
    double* _noiseLast2;

    double* _noiseFirst3;
    double* _noiseCurrent3;
    double* _noiseLast3;

};

#endif
