// This class bundles up all parameters for the predictors. Most of
// these parameters will come from the UI, other may come from files,
// or explicitly defined.  Each predictor knows which params it needs.

#ifndef PBUNDLE_H
#define PBUNDLE_H

#include "types.H"

class ParamBundle 
{
  public:
    ParamBundle() { reset(); }
    virtual ~ParamBundle() { }

  void reset() {
    _degree = 0;
    _numpts = 0;
    _specd = 0;
    _cispecd = 0;
    _mmaespecd = 0;
    _leadtime = 0;
    _lptype = tNone3;
    _estype = tNone4;
    _ktype  = tNone5;
    _citype = tNone6;
    _mmaetype = tNone7;
    _cmbtype = tSUM;
    _derfly = 0;
    _kfflag = 0;
    _kfexp = 0;
    _kfgrey = 0;
    _srate = 0.0;
    _alpha = 0.01;
    _beta  = 0.01;
    _greypts = 4;
    _origsrate = 0.0;
    _posvar = 0.0;
    _quatvar = 0.0;
    _ukfon = 0.0;
    _seperator = 1;
  }

  // acessors 
  int           getDegree()        { return _degree; }
  int           getNumPts()        { return _numpts; }
  int           getLeadTime()      { return _leadtime; } 
  int           getSpecD()         { return _specd; }
  int           getCISpecD()       { return _cispecd; }
  int           getMMAESpecD()     { return _mmaespecd; }
  KalmanType    getKType()         { return _ktype; }
  LocalPolyType getLPolyType()     { return _lptype; }
  ExpSmoothType getExpSmoothType() { return _estype; } 
  CIType        getCIType()        { return _citype; }
  MMAEType      getMMAEType()      { return _mmaetype; } 
  CombinerType  getCombiner()      { return _cmbtype; }
  int           getDerFly()        { return _derfly; }
  int           getKFFlag()        { return _kfflag; }
  double        getCurSRate()      { return _srate; }
  double        getAlpha()         { return _alpha; }
  double        getBeta()          { return _beta; }
  int           getGreyPts()       { return _greypts; }
  double        getOrigSRate()     { return _origsrate; }
  double        getPosVar()        { return _posvar; }
  double        getQuatVar()       { return _quatvar; }
  int           getUKFFlag()       { return _ukfon; }
  int           getKFExp()         { return _kfexp; }
  int           getKFGrey()        { return _kfgrey; }
  int           getSPSep()         { return _seperator; }

  void setDegree(int d)    { _degree = d; }
  void setNumPts(int np)   { _numpts = np; }
  void setLeadTime(int l)  { _leadtime = l; }
  void setSpecD(int n)     { _specd = n; }
  void setCISpecD(int n)   { _cispecd = n; }
  void setMMAESpecD(int n) { _mmaespecd = n; }
  void setKType(KalmanType kt)            { _ktype = kt; }
  void setLPolyType(LocalPolyType tp)     { _lptype = tp; }
  void setExpSmoothType(ExpSmoothType te) { _estype = te; }
  void setCIType(CIType ci)               { _citype = ci; }
  void setMMAEType(MMAEType m)            { _mmaetype = m; }
  void setCombiner(CombinerType cb)       { _cmbtype = cb; }
  void setDerFly(int f)       { _derfly = f; }  
  void setKFFlag(int k)       { _kfflag = k; }
  void setCurSRate(double s)  { _srate = s; }
  void setAlpha(double a)     { _alpha = a; }
  void setBeta(double b)      { _beta = b; }
  void setGreyPts(int g)      { _greypts = g; }
  void setOrigSRate(double s) { _origsrate = s; }
  void setPosVar(double pv)   { _posvar = pv; }
  void setQuatVar(double qv)  { _quatvar = qv; }
  void setUKFFlag(int ukf)    { _ukfon = ukf; }  
  void setKFExp(int kf)       { _kfexp = kf; } 
  void setKFGrey(int kf)      { _kfgrey = kf; }
  void setSPSep(int sep)      { _seperator = sep; }

  protected:
    LocalPolyType  _lptype;
    ExpSmoothType  _estype;
    KalmanType     _ktype;
    CIType         _citype;
    MMAEType       _mmaetype;
    CombinerType   _cmbtype;
    int            _mmaespecd;
    int            _cispecd; 
    int            _specd;
    int            _degree;
    int            _numpts;
    int            _leadtime;
    int            _derfly;
    int            _kfflag;
    int            _kfexp;
    int            _kfgrey;
    int            _greypts;
    double         _srate;
    double         _alpha;
    double         _beta;
    double         _origsrate;
    double         _posvar;
    double         _quatvar;
    int            _ukfon;
    int            _seperator;
    

   
};



#endif
