// base classes for MMAE predictors
// NOTE: these classes should be instanciated, not inherited

#ifndef MMAEBASE_H
#define MMAEBASE_H

#include "mmatrix.H"
#include "../pose.H"
#include "kalmanUpdate.H"

#define MAXBEGIN 16
#define INITVAR 100


class MMAEKalmanBase
{
 public: 
   MMAEKalmanBase() { }
   MMAEKalmanBase(KALMANMODEL mod);
   virtual ~MMAEKalmanBase();

   void initialize(double spd, double dt, double noisep);
   void mmaeKalmanFilter(Pose *userpos, int zNum);
   void resetPP(); 

   // acessors 
   MMatrix & getResidual()    { return _residual; }
   MMatrix & getResidualCov() { return _AA; }
   MMatrix & getStateVector() { return _state; }
   MMatrix & getFundMat()     { return _fundMat; }
   MMatrix & getCov()         { return _PP; }
   

   MMAEKalmanBase & operator= ( const MMAEKalmanBase &m ) {
     _mod      = m._mod;
     _HH       = m._HH; 
     _QQ       = m._QQ;
     _RR       = m._RR;
     _fundMat  = m._fundMat;
     _AA       = m._AA;
     _residual = m._residual;
     _PP       = m._PP;
     _state    = m._state;
    
     return *this;
   }

 protected:
   KALMANMODEL _mod;   

   MMatrix  _HH;
   MMatrix  _QQ;
   MMatrix  _RR;
   MMatrix  _fundMat;
   MMatrix  _AA;
   MMatrix  _residual;
   MMatrix  _PP; 
   MMatrix  _state;    
 
};


class MMAEKalmanBaseCompWise
{
 public:  
   MMAEKalmanBaseCompWise() { }
   MMAEKalmanBaseCompWise(KALMANMODEL mod);
   virtual ~MMAEKalmanBaseCompWise();

   void initialize(double spd, double dt, double noisep);
   void mmaeKalmanFilter(Pose *userpos, int zNum);

   // acessors 
   MMatrix & getResidual(int i)    { return _residual[i]; }
   MMatrix & getResidualCov(int i) { return _AA[i]; }
   MMatrix & getStateVector(int i) { return _state[i]; }
   MMatrix & getFundMat()     { return _fundMat; }
   MMatrix & getCov(int i)         { return _PP[i]; }

   MMAEKalmanBaseCompWise & operator= ( const MMAEKalmanBaseCompWise &m ) {
     _mod      = m._mod;
     _HH       = m._HH; 
     _QQ       = m._QQ;
     _RR       = m._RR;
     _fundMat  = m._fundMat;
     
     for (int i = 0; i < 3; ++i) {
	_AA[i]       = m._AA[i];
	_residual[i] = m._residual[i];
	_PP[i]       = m._PP[i];
	_state[i]    = m._state[i];
     }
     return *this;
   }

 protected:
    KALMANMODEL _mod;   

    MMatrix  _HH;
    MMatrix  _QQ;
    MMatrix  _RR;
    MMatrix  _fundMat;
    MMatrix  _AA[3];
    MMatrix  _residual[3];
    MMatrix  _PP[3]; 
    MMatrix  _state[3];    
 

};



class MMAEExtendedKalmanBase 
{
 public: 
   MMAEExtendedKalmanBase() { }
   MMAEExtendedKalmanBase(KALMANMODEL mod);
   virtual ~MMAEExtendedKalmanBase();

   void initialize(double spd, double dt, double noiseq);
   void mmaeExtendedKalmanFilter(Pose *userpos, int zNum);
   void resetPP(); 

   // acessors 
   MMatrix & getResidual()    { return _Eresidual; }
   MMatrix & getResidualCov() { return _EA; }
   MMatrix & getStateVector() { return _Estate; }
   MMatrix & getFundMat()     { return _EfundMat; }
   MMatrix & getCov()         { return _EP; }
   

   MMAEExtendedKalmanBase & operator= ( const MMAEExtendedKalmanBase &m ) {
     _mod      = m._mod;
     _EH       = m._EH; 
     _EH2      = m._EH2;
     _EQ       = m._EQ;
     _ER       = m._ER;
     _EfundMat = m._EfundMat;
     _EA       = m._EA;
     _Eresidual= m._Eresidual;
     _EP       = m._EP;
     _Estate   = m._Estate;
    
     return *this;
   }

 protected:
   KALMANMODEL _mod;   

   MMatrix  _EH;
   MMatrix  _EH2;
   MMatrix  _EQ;
   MMatrix  _ER;
   MMatrix  _EfundMat;
   MMatrix  _EA;
   MMatrix  _Eresidual;
   MMatrix  _EP; 
   MMatrix  _Estate;    

   double _edt;
   double _spd;
  
   Updater _up;
};


#endif
