// Joseph J LaViola Jr.
// Basic Matrix class


#ifndef MATRIX_H
#define MATRIX_H

#include <iostream.h>
#include <stdio.h>
#include "vvector.H"
#include "matrix.h"

class MMatrix {
  protected:
   double  **_matrix;
   int       _NN;
   int       _MM;
  // simple initializer
  void initializeMatrix()
  {
   for (int i = 0; i < _MM; ++i)
     for (int j = 0; j < _NN; ++j)
       _matrix[i][j] = 0.0;
  }

  void createIdentity()
  {
    initializeMatrix();
    for (int i = 0; i < _MM; ++i)
       _matrix[i][i] = 1.0;
  }

  public:
  MMatrix(int m, int n) 
   {   
     _matrix = 0;
     _NN = n;
     _MM = m;
     allocateMatrix(m,n);
   }

   MMatrix(int n) // used to make a square matirix initialized to identity
   {
    _NN = n;
    _MM  = n;
    allocateMatrix(n,n);
    createIdentity();
   }

   MMatrix() : _MM(0), _NN(0), _matrix(0) {}
 
   MMatrix(MMatrix *m, double scale = 1.0) {
     m->getDim(&_MM, &_NN);
     allocateMatrix(_MM,_NN);
     for (int i = 0; i < _MM; ++i)
      for (int j = 0; j < _NN; ++j)
       _matrix[i][j] = ((*m)(i,j) * scale);
   }
    
   virtual ~MMatrix() { deleteMatrix(); }

   void allocateMatrix(int m, int n) 
   {
     _NN = n;
     _MM = m;
     _matrix = new double*[m];
     for (int i=0; i < m; ++i)
       _matrix[i] = new double[n];
     initializeMatrix();
   }

   void deleteMatrix()
   { 
     for (int i=0; i < _MM; ++i)
        delete [] _matrix[i];
     delete [] _matrix;
     _NN = 0;
     _MM = 0;
     _matrix = 0;
   }

   void setIdentity()
   {
     for (int i = 0; i < _MM; ++i)
        _matrix[i][i] = 1.0;
   }

   void setZeros() 
   { 
    for (int i = 0; i < _MM; ++i)
     for (int j = 0; j < _NN; ++j)
       _matrix[i][j] = 0.0;
   }  

  
  MMatrix & operator= ( const MMatrix &m ) {
    if (_matrix == 0) {
      _MM = m._MM;
      _NN = m._NN;
      _matrix = new double*[_MM];
      for (int i = 0; i < _MM; ++i) {
	_matrix[i] = new double[_NN];
	for (int j = 0; j < _NN; ++j)
	  _matrix[i][j] = m._matrix[i][j];
      }      
    }
    else { // just copy 
       for (int i = 0; i < _MM; ++i)
	for (int j = 0; j < _NN; ++j)
	  _matrix[i][j] = m._matrix[i][j];
    }
    
    return *this;
  }

   MMatrix( const MMatrix &m ) : _MM(0), _NN(0), _matrix(0) {
      *this = m;
   }

   // some overloaded operators
   double operator () (int i, int j) const
   {  
    return _matrix[i][j];
   }

   double & operator () (int i, int j)
   {
    return _matrix[i][j];
   }

   const MMatrix & operator+= (const MMatrix &a) {
     int m,n;
     a.getMDim(&m, &n);
     assert(m == _MM);
     assert(n == _NN);
     for (int i = 0; i < _MM; ++i)
       for (int j = 0; j < _NN; ++j)
         _matrix[i][j] += a(i,j);
     return *this;
   }     

   // assumes matrices are of the same dimension
   MMatrix * addMatrix(MMatrix *m) {
     
     MMatrix *newmat = new MMatrix(_MM,_NN);
     for (int i = 0; i < _MM; ++i)
       for (int j = 0; j < _NN; ++j)
         (*newmat)(i,j) = _matrix[i][j] + (*m)(i,j);

    return newmat;
   }  

   void getDim(int *m, int *n) { *m = _MM; *n = _NN; }
   void getMDim(int *m, int *n) const { *m = _MM; *n = _NN; }

   MMatrix * getMatrix() {return this;}


   VVector * multVec(VVector *v) {
     // make sure the vector and matrix can multiply
     if (v->getSize() != _NN) {
       cerr << "Columns of matrix do not match up with column vector" << endl;
       return NULL;
     }
     VVector *nv = new VVector(_NN);
     for (int i = 0; i < _MM; ++i)
       for (int j = 0; j < _NN; ++j)
          (*nv)[i] += _matrix[i][j] * (*v)[j];  
     
     return nv;
   }

   void multScalar(double s) {
     for (int i = 0; i < _MM; ++i)
       for (int j = 0; j < _NN; ++j)
         _matrix[i][j] *= s;
   }

   MMatrix * transpose() {
     MMatrix *tmp = new MMatrix(_NN, _MM);

     for (int i = 0; i < _MM; ++i)
       for (int j = 0; j < _NN; ++j)
         (*tmp)(j,i) = _matrix[i][j];
     return tmp; 
   }
    
      
  MMatrix getColumn(int i) {
    if ((i < _NN) && (i >= 0)) {
     MMatrix tmp(_MM,1);
     for (int j = 0; j < _MM; ++j)
        tmp(j,0) = _matrix[j][i];
     return tmp;
    }
    else {
      cerr << "Problem in getColumn!" << endl;
      return 0;
    }
  }

  int insertColumn(const MMatrix &a, int i) {
    if ((i < _NN) && (i >= 0)) {
     for (int j = 0; j < _MM; ++j)
       _matrix[j][i] = a(j,0);
     return 1;
    }
    else 
     return 0;
  }

   // nice print routine
   void printMatrix() 
   {
    cerr << "---------------MMatrix----------------" << endl;
    for (int i = 0; i < _MM; ++i) {
      for (int j = 0; j < _NN; ++j)
       fprintf(stderr, "%12.9lf ", _matrix[i][j]);
         cerr << endl;
    } 
    cerr << endl;
   }

};

inline MMatrix *
mmult(MMatrix *a, MMatrix *b) {
  int n,m,p,q;
  a->getDim(&m,&n);
  b->getDim(&p,&q);
  assert(n == p);

  MMatrix *nm = new MMatrix(m,q);
 
  for (int i = 0; i < m; ++i)
    for (int j = 0; j < q; ++j)
      {
        (*nm)(i,j) = 0.0;
        for (int k = 0; k < n; ++k)
          (*nm)(i,j) += (*a)(i,k) * (*b)(k,j);
      }
  return nm;  
}


// some overloaded operators which will make my life easier

inline MMatrix
operator * (const MMatrix &a, const MMatrix &b) {
  int n,m,p,q;
  a.getMDim(&m,&n);
  b.getMDim(&p,&q);
  assert(n == p);

  MMatrix nm(m,q);
  for (int i = 0; i < m; ++i)
    for (int j = 0; j < q; ++j) {
      nm(i,j) = 0.0;
      for (int k = 0; k < n; ++k)
	nm(i,j) += a(i,k) * b(k,j);
    }

  return nm;
}

inline MMatrix
operator + (const MMatrix &a, const MMatrix &b) {
  int n,m,p,q;
  a.getMDim(&m,&n);
  b.getMDim(&p,&q);
  assert(m == p);
  assert(n == q);

  MMatrix nm(m,n);
  for (int i = 0; i < m; ++i)
    for (int j = 0; j < n; ++j)
      nm(i,j) = a(i,j) + b(i,j);
  
  return nm;
}

inline MMatrix
operator - (const MMatrix &a, const MMatrix &b) {
  int n,m,p,q;
  a.getMDim(&m,&n);
  b.getMDim(&p,&q);
  assert(m == p);
  assert(n == q);

  MMatrix nm(m,n);
  for (int i = 0; i < m; ++i)
    for (int j = 0; j < n; ++j)
      nm(i,j) = a(i,j) - b(i,j);
  
  return nm;
}

inline MMatrix
transp (const MMatrix &a) {
  int m,n;
  a.getMDim(&m,&n);
  MMatrix tmp(n, m);

  for (int i = 0; i < m; ++i)
    for (int j = 0; j < n; ++j)
      tmp(j,i) = a(i,j);

  return tmp; 
}


inline MMatrix
operator * (double s, const MMatrix &a) {
  int m,n;
  a.getMDim(&m,&n);

  MMatrix tmp(m,n);
   for (int i = 0; i < m; ++i)
    for (int j = 0; j < n; ++j)
      tmp(i,j) = s * a(i,j);
  
 return tmp;
}


// faster version but it does have a bug
inline MMatrix
minverse (const MMatrix &a) {

 int m,n; 
 a.getMDim(&m,&n);
    
 // if matrix is 1 by 1 it is just the scalar inverse
 if ((m == 1) && (n == 1)) {
  MMatrix tmp(1,1);
  tmp(0,0) = 1.0 / a(0,0);
  return tmp;
 }

 MMatrix Inv(n,n);
 matrix M1(n,n);
 for (int r = 0; r < n ; ++r)
   for (int s = 0; s < n; ++s)
     M1.setvalue(r,s, a(r,s)); 
    
 M1.invert();
 for (int r = 0; r < n; ++r)
   for (int s = 0; s < n; ++s) {
     double g;
     int p;
     M1.getvalue(r,s, g, p);  
     Inv(r,s) = g;
   }
// M1.destroy();
 return Inv;

}


// slower version but does not suffer from the bug that minverse has
inline MMatrix
minverse2 (const MMatrix &a) {
  // we LU decompse first
  int n = 0;
  int m = 0;
  a.getMDim(&m,&n);

 // if matrix is 1 by 1 it is just the scalar inverse
 if ((m == 1) && (n == 1)) {
  MMatrix tmp(1,1);
  tmp(0,0) = 1.0 / a(0,0);
  return tmp;
 }

  // create the L and U matrices
  MMatrix L(n,n);
  MMatrix U(n,n);

  for (int k = 0; k < n; ++k) {
    L(k,k) = 1.0;
    for(int j = k; j < n; ++j) {
      double sum = 0.0;
      for(int s = 0; s < k; ++s) {
        sum += L(k,s) * U(s,j);
      }
      U(k,j) = a(k,j) - sum;
    }
    for (int i = k + 1; i < n; ++i) {
      double sum = 0.0;
      for (int s = 0; s < k; ++s)
	sum += L(i,s) * U(s,k);
      L(i,k) = (a(i,k) - sum) / U(k,k);
    }
  }
  
  
  // now we find the inverse by computing a series of linear solves
  MMatrix tmp(n,n);
  VVector *b = new VVector(n);
  
  for (int p = 0; p < n; ++p) {
    for (int q = 0; q < n; ++q)
      (*b)[q] = 0.0;
    (*b)[p] = 1.0;   
   
    // 	first we solve Lz = b
    VVector *z = new VVector(n);
    VVector *x = new VVector(n);
    for (int i = 0; i < n; ++i) {
      double sum = 0.0;
      for (int j = 0; j < i; ++j)
	sum += L(i,j) * (*z)[j];
      (*z)[i] = ((*b)[i] - sum) / L(i,i);
    }
    // 	now solve Ux = z
    for (int i = n-1;  i >= 0; --i) {
      double sum = 0.0;
      for (int j = i; j < n; ++j)
	sum += U(i,j) * (*x)[j];
      (*x)[i] = ((*z)[i] - sum) / U(i,i);
    }
    // x is now a column of the inverse matrix
    for (int q = 0; q < n; ++q)
      tmp(q,p) = (*x)[q];

    delete z;
    delete x;
  }
    
  delete b;  
 
  return tmp;
}


// square root of a matrix using cholesky decompositions
// assumes matrix must be symmetric, positive definite
inline MMatrix
choleskyDecomp(const MMatrix &a)
{
 int n = 0; 
 a.getMDim(&n,&n);

 MMatrix l(n,n);

 for (int k = 1; k <= n; ++k) {
  double sum1 = 0;
  for (int s = 1; s <= k - 1; ++s)
     sum1 += l(k-1,s-1) * l(k-1,s-1);

  l(k-1,k-1) = sqrt(a(k-1,k-1) - sum1);

  for (int i = k+1; i <= n; ++i) {
    double sum2 = 0;
    for (int s2 = 1; s2 <= k-1; ++s2)
      sum2 += l(i-1,s2-1) * l(k-1, s2-1);
    l(i-1,k-1) = (a(i-1,k-1) - sum2) / l(k-1,k-1);
       
  }
 }
 return l;
}


// compute the determinant of a matrix
// do a LU decomp and multiply diagonals
inline double
det(const MMatrix &a)
{
  int n = 0;
  int m = 0;
  a.getMDim(&n,&m);
  assert(m == n);

  if (m == 1)
    return a(0,0);

  // create the L and U matrices
  MMatrix L(n,n);
  MMatrix U(n,n);

  for (int k = 0; k < n; ++k) {
    L(k,k) = 1.0;
    for(int j = k; j < n; ++j) {
      double sum = 0.0;
      for(int s = 0; s < k; ++s) {
        sum += L(k,s) * U(s,j);
      }
      U(k,j) = a(k,j) - sum;
    }
    for (int i = k + 1; i < n; ++i) {
      double sum = 0.0;
      for (int s = 0; s < k; ++s)
	sum += L(i,s) * U(s,k);
      L(i,k) = (a(i,k) - sum) / U(k,k);
    }
  }

  double res = 1.0;
  for (int i = 0; i < n; ++i) 
    res *= U(i,i);

  return res;
}

#endif
