// member function definitions for the UI classes

#include "ui.H"
#include "pose.H"
#include "dispenser.H"
#include "sanalyzer.H"
#include "ndifftools.H"

#include <qapplication.h>
#include <iostream.h>
#include <string.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qlayout.h>
#include <qnamespace.h> 
#include <qlayout.h>
#include <qmultilineedit.h>
#include <qstring.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qcursor.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qprogressdialog.h>

const double NOISED = 1000000.0;
const double EXPPARAM = 100.0;

// SLOTS for QLCDNumberD
void
QLCDNumberD::myDisplay(int num)
{
  // turn num into a double
  if (num == 0)
   display(0.0); 
  else
   display((double)num/NOISED);
}

void
QLCDNumberD::myDisplay2(int num)
{
  // turn num into a double
  if (num == 0)
   display(0.01); 
  else
   display((double)num/EXPPARAM);
}

void 
QLCDNumberD::myDisplay3(int num)
{
 display(num*10);
}



// constructor for Global Widget class

GlobalWidget::GlobalWidget(QWidget *parent, const char *name) : QWidget(parent, name)
                                                                
{
  // set the size of the window
  this->setGeometry(0,0,800,700);

  // create the menu bar and menu items
  _menu = new QMenuBar(this, "MenuBar");
  _menu->setSeparator(QMenuBar::InWindowsStyle);
  _filemenu = new QPopupMenu;
  _filemenu->insertItem("Load Dataset", this, SLOT( loadFile() ));
  _filemenu->insertItem("Save Log", this, SLOT( saveLog() )); 
  _filemenu->insertItem("Quit", qApp, SLOT( closeAllWindows() ));
  connect( qApp, SIGNAL( lastWindowClosed() ), this, SLOT( myquit() ));
  _menu->insertItem("File", _filemenu);
  
  _predictor = new QPopupMenu;
  _predictor->insertItem("Reflection", this, SLOT( reflection() ));
  _predictor->insertItem("Local Polynomial", this, SLOT( localPolynomial() ));
  _predictor->insertItem("Exponetial Smoothing", this, SLOT( expSmoothing() ));
  _predictor->insertItem("Grey System Prediction", this, SLOT( greyPred() ));
  _predictor->insertItem("Kalman Filter-Based", this, SLOT( kalmanPred() ));
  _predictor->insertItem("Covariance Intersection", this, SLOT( ciPred() ));
  _predictor->insertItem("Multiple Model Adaptive Estimation", this, SLOT( mmaePred() ));
  _menu->insertItem("Choose Predictor", _predictor);  

  _errormet = new QPopupMenu;
  _errormet->insertItem("Root MSE", this, SLOT( rootMSE() ));
  _errormet->insertItem("L-Infinity", this, SLOT( lInfinity() ));
  _errormet->insertItem("Percent Better", this, SLOT( pctBetter() ));
  _errormet->insertItem("Choose All", this, SLOT( chooseAll() ));
  _menu->insertItem("Error Metric", _errormet);  

  _testing = new QPopupMenu;
  _testing->insertItem("Single Run", this, SLOT( singleRun() ));
  _testing->insertItem("Run over Algorithm Params", this, SLOT( runParams() ));
  _testing->insertItem("Run over Sampling Rates", this, SLOT( runSampleRate() ));
  _testing->insertItem("Run over Prediction Interval", this, SLOT( runPredictInt() ));
  _testing->insertItem("Run over Noise", this, SLOT( runNoise() ));
  _testing->insertItem("Run Full Test", this, SLOT( runFull() ));
  _menu->insertItem("Test Predictor", _testing);

  _mlab = new QPopupMenu;
  _mlab->insertItem("Start Matlab", this, SLOT( startMatLab() ));
  _mlab->insertItem("Show Plot", this, SLOT( showPlot() ));
  _mlab->insertItem("Launch SPTool", this, SLOT( spTool() ));
  _menu->insertItem("MatLab", _mlab);

  _siga = new QPopupMenu;
  _siga->insertItem("Analyzers", this, SLOT( analyzers() ));
  _siga->insertItem("Numerical D", this, SLOT( nDiffTools() ));
  _menu->insertItem("Signal Analysis", _siga);
  
  // add statusbar
  _status = new QStatusBar(this, "StatusBar");
  _status->message("Welcome to JJL's Predictor Tester");

  // create the textbox

  _textbox = new QMultiLineEdit(this, "textbox");
  _textbox->setReadOnly(TRUE);
 
  // create the control panel
  _cpanel = new ControlPanel(this, "CPanel");

  QVBoxLayout *layout = new QVBoxLayout(this);
  layout->setMenuBar(_menu);
  layout->addWidget(_cpanel);
  layout->addWidget(_textbox);
  layout->addWidget(_status);

  // create dialogs
  _plotsd    = new PlotsDialog(0, "Plot Dialog");
  _lpolyd    = new LocalPolyDialog(0, "Polynomial Predictor Dialog");
  _expd      = new ExpSmoothDialog(0, "Exponential Smoothing Dialog");
  _greyd     = new GreyPredDialog(0, "Grey System Predictor Dialog");
  _kpredd    = new KalmanPredDialog(0, "Kalamn Filter Predictor Dialog");
  _cipredd   = new CIPredDialog(0, "Covariance Intersection Dialog");
  _mmaepredd = new MMAEPredDialog(0, "MMAE Predictor Dialog");
  _sigad = new SignalAnalysisDialog(this, 0, "Signal Analysis Dialog");
  _ndiffd = new NDiffToolDialog(this, 0, "Numerical Differentation Dialog");

  // create the process dialog
  _processd = new QProgressDialog(0,"Progress Dialog",TRUE);

  // create an empty user motion record
  _urec = new UMotionRec();
  // create matlab engine object
  _mengine = new MATLABEngine();
  _ptype = tNone;
  _etype = tNone1;
  _rtype = tNone2;
  _matlabon = 0;
  _dataready = 0;
  _datapresent = 0;

  // create the dispenser object
  _disp = new Dispenser(this);

}


// destructor
GlobalWidget::~GlobalWidget()
{
   delete _plotsd;
   delete _lpolyd;
   delete _expd;
   delete _greyd;
   delete _kpredd;
   delete _cipredd;
   delete _mmaepredd;
   delete _sigad;
   delete _ndiffd;
   delete _processd;
   delete _urec;
   delete _mengine;
   delete _status;
   delete _cpanel;
   delete _textbox;
   delete _filemenu;
   delete _predictor;
   delete _testing;
   delete _errormet;
   delete _mlab;
   delete _siga;
   delete _menu;
   delete _disp;
}

// slot implementations for GLOBAL WIDGET

// this function will create a file dialog box to load a user motion data set
void
GlobalWidget::loadFile() 
{
  QString fn = QFileDialog::getOpenFileName("../data/work_data/", QString::null,
					     this);
  if ( !fn.isEmpty() ) {
    QApplication::setOverrideCursor(Qt::waitCursor);
    loadMotionData(fn.latin1(), fn.length());
    QApplication::restoreOverrideCursor();
    _dataready = 1;
  }
  else
    _status->message( "Loading aborted", 4000 ); 

}

// saves all of the info in the textbox to a file
void
GlobalWidget::saveLog()
{
    QString fn = QFileDialog::getSaveFileName("../data/logs/", QString::null,
					     this);
    if ( !fn.isEmpty() ) {
      saveLogFile(fn);
    } else {
      _status->message( "Saving aborted", 4000 );
    }

}

// quits the program
void
GlobalWidget::myquit()
{
  // close down the Matlab engine if it is running
  if (_mengine->getEngine()) 
    _mengine->killEngine();

  QApplication::exit(0);
}


// PREDICTOR SLOTS
// these slots are for predictors, they will trigger new windows with
// additonal options and parameter adjusters

void 
GlobalWidget::reflection()
{
 _ptype = tReflexive;
 setPredictor();
 _lpolyd->hideDialog();
 _expd->hideDialog();
 _greyd->hideDialog();
 _kpredd->hideDialog();
 _cipredd->hideDialog();
 _mmaepredd->hideDialog();
}

void
GlobalWidget::localPolynomial()
{
  _ptype = tLocalPoly;
  setPredictor();
  _lpolyd->showDialog();
  _expd->hideDialog();
  _greyd->hideDialog();
  _kpredd->hideDialog();
  _cipredd->hideDialog();
  _mmaepredd->hideDialog();
}

void 
GlobalWidget::expSmoothing()
{
  _ptype = tExpSmooth;
  setPredictor();
  _lpolyd->hideDialog();
  _expd->showDialog();
  _greyd->hideDialog();
  _kpredd->hideDialog();
  _cipredd->hideDialog();
  _mmaepredd->hideDialog();
}


void
GlobalWidget::greyPred()
{
  _ptype = tGrey;
  setPredictor();
  _lpolyd->hideDialog();
  _expd->hideDialog();
  _greyd->showDialog();
  _kpredd->hideDialog();
  _cipredd->hideDialog();
  _mmaepredd->hideDialog();
}

void
GlobalWidget::kalmanPred()
{
 _ptype = tKalman;
 setPredictor();
 _lpolyd->hideDialog();
 _expd->hideDialog();
 _greyd->hideDialog();
 _cipredd->hideDialog();
 _kpredd->showDialog();
 _mmaepredd->hideDialog();

}

void
GlobalWidget::ciPred()
{
 _ptype = tCI;
 setPredictor();
 _lpolyd->hideDialog();
 _expd->hideDialog();
 _greyd->hideDialog();
 _kpredd->hideDialog();
 _cipredd->showDialog();
 _mmaepredd->hideDialog();
}

void 
GlobalWidget::mmaePred()
{
 _ptype = tMMAE;
 setPredictor();
  _lpolyd->hideDialog();
 _expd->hideDialog();
 _greyd->hideDialog();
 _kpredd->hideDialog();
 _cipredd->hideDialog();
 _mmaepredd->showDialog();

}

// ERROR METRIC SLOTS
// these slots are used to deternime what error metrics to utilize
// when evaluating predictors
void 
GlobalWidget::rootMSE()
{
  _etype = tRootMSE;
  _cpanel->getCurError()->setText("Root MSE");

}

void
GlobalWidget::lInfinity()
{

  _etype = tLInfinity;
  _cpanel->getCurError()->setText("L-Infinity");
}


void
GlobalWidget::pctBetter()
{
 _etype = tPctBetter;
 _cpanel->getCurError()->setText("Pct. Better");
}

void
GlobalWidget:: chooseAll()
{
  _etype = tChooseAll;
  _cpanel->getCurError()->setText("All");
}


// TESTING SLOTS
// use to run a particular prediciton algorithm using the given test method
void
GlobalWidget::singleRun()
{
  QApplication::setOverrideCursor(Qt::waitCursor);
 _rtype = tSingle;
 if (!_datapresent)
   _status->message("No data set loaded.", 5000); 
 else if (_etype == tNone1)
   _status->message("No error metric selected.", 5000); 
 else if (_ptype == tNone)
   _status->message("No predictor selected.", 5000); 
 else {
   _disp->collectTestInfo();  // get all testing parameter info first
   _disp->runSingleTest();    // run the test and report results
 }
 QApplication::restoreOverrideCursor();
}


void 
GlobalWidget::runParams()
{
  QApplication::setOverrideCursor(Qt::waitCursor);
 _rtype = tRunParams;
 if (!_datapresent)
   _status->message("No data set loaded.", 5000); 
 else if (_etype == tNone1)
   _status->message("No error metric selected.", 5000); 
 else if (_ptype == tNone)
   _status->message("No predictor selected.", 5000); 
 else {
   _disp->collectTestInfo();  // get all testing parameter info first
   _disp->runParamTest();     // run the test and report results
 }
 QApplication::restoreOverrideCursor();
}

void
GlobalWidget::runSampleRate()
{
  QApplication::setOverrideCursor(Qt::waitCursor);
 _rtype = tRunSP;
 if (!_datapresent)
   _status->message("No data set loaded.", 5000); 
 else if (_etype == tNone1)
   _status->message("No error metric selected.", 5000); 
 else if (_ptype == tNone)
   _status->message("No predictor selected.", 5000); 
 else {
   _disp->collectTestInfo();  // get all testing parameter info first
   _disp->runOverSRate();
 }
 QApplication::restoreOverrideCursor();
}


void
GlobalWidget::runPredictInt()
{
  QApplication::setOverrideCursor(Qt::waitCursor);
  _rtype = tRunPI;
  if (!_datapresent)
    _status->message("No data set loaded.", 5000); 
  else if (_etype == tNone1)
    _status->message("No error metric selected.", 5000); 
  else if (_ptype == tNone)
    _status->message("No predictor selected.", 5000); 
  else {
    _disp->collectTestInfo();  // get all testing parameter info first
    _disp->runOverPInt();
  }
  QApplication::restoreOverrideCursor();
}

void
GlobalWidget::runNoise()
{
 QApplication::setOverrideCursor(Qt::waitCursor);
 _rtype = tRunN;
  if (!_datapresent)
    _status->message("No data set loaded.", 5000); 
  else if (_etype == tNone1)
    _status->message("No error metric selected.", 5000); 
  else if (_ptype == tNone)
    _status->message("No predictor selected.", 5000); 
  else {
    _disp->collectTestInfo();  // get all testing parameter info first
    _disp->runOverErrVar();
  }
  QApplication::restoreOverrideCursor();
}


void
GlobalWidget::runFull()
{

 QApplication::setOverrideCursor(Qt::waitCursor);
 _rtype = tRunAll;
 if (!_datapresent)
   _status->message("No data set loaded.", 5000); 
 else if (_etype == tNone1)
   _status->message("No error metric selected.", 5000); 
 else if (_ptype == tNone)
   _status->message("No predictor selected.", 5000); 
 else {
    _disp->collectTestInfo();  // get all testing parameter info first
    _disp->runOverAll();
 }
  QApplication::restoreOverrideCursor();
 
}


// MATLAB SLOTS
// this slots are used to call matlab functions, mostly graphics
void 
GlobalWidget::startMatLab()
{ 
  if (!_matlabon) { 
    QApplication::setOverrideCursor(Qt::waitCursor);
    int okay = _mengine->startEngine();
    QApplication::restoreOverrideCursor();
    if (okay) {
      _matlabon = 1;
      _status->message("Matlab Engine Running...", 3000);
      _textbox->append("Matlab Engine Active.");
    }
    else
      _status->message("Problem Starting MatLab...");
  }
  else
   _status->message("Matlab Engine already active");
}

void
GlobalWidget::showPlot()
{
 _plotsd->showDialog();
}


void
GlobalWidget::spTool()
{

  QApplication::setOverrideCursor(Qt::waitCursor);
  int okay = _mengine->runCmd(QString("sptool"));
  QApplication::restoreOverrideCursor();  
  if (okay) {
    _status->message("Matlab SPTool Launched", 4000);
    if (_dataready) { // if we have a motion record, load data into Matlab
      QApplication::setOverrideCursor(Qt::waitCursor);
      sendUserDataToMatlab();
      QApplication::restoreOverrideCursor();
      _status->message("User Motion data sent to Matlab" , 4000);
      _dataready = 0;
    }
  }
  else
    _status->message("Problem launching SPTool.  Make sure Matlab Engine is running.");
}


// SIGNAL ANALYSIS SLOTS
// used to run the different signal analysis and quantification routines
void
GlobalWidget::analyzers()
{
 _ndiffd->hideDialog();
 _sigad->showDialog();
}


void
GlobalWidget::nDiffTools()
{
 _sigad->hideDialog();
 _ndiffd->showDialog();  
}


//////////////////////////////////////////////////
// non-slot functions
void
GlobalWidget::setPredictor()
{
  switch(_ptype) {
    case tReflexive:   
      _cpanel->getCurPredictor()->setText("Reflection");
      break;              
    case tLocalPoly: 
      _cpanel->getCurPredictor()->setText("Local Polynomial");
      break;
    case tExpSmooth: 
      _cpanel->getCurPredictor()->setText("Exp. Smoothing");
      break;
    case tGrey:  
      _cpanel->getCurPredictor()->setText("Grey System");
      break;
    case tKalman:   
      _cpanel->getCurPredictor()->setText("Kalman Filter");
      break;   
    case tCI: 
      _cpanel->getCurPredictor()->setText("Cov. Intersection");
      break;        
    case tMMAE:
      _cpanel->getCurPredictor()->setText("MMAE");
      break;
    default:
      cerr << "Problem if we get here." << endl;
  }

}
void 
GlobalWidget::loadMotionData(const char *file, int length)
{
  // load the data in the the urec
  // first kill the old rec
  delete _urec;
 _urec = new UMotionRec();  

  _urec->loadData(file);
  
  int num = 0;
  for (int i = 0; i < length; ++i) 
    if (file[i] == '/')
      num = i;
  char name[100];
  int j = 0;
  for (j = num+1; j < length; ++j) 
    name[j-(num+1)] = file[j];
  name[j-(num+1)] = '\0'; 
  _status->message("DataSet: " + QString(name) + " loaded.", 5000);
  _textbox->append(QString(name) + " is the active data set.");
  _datapresent = 1;
 
 // update other UI components
  _cpanel->getCurFile()->setText(QString(name));
  _cpanel->getOrigSP()->setNum(_urec->getSamplingRate());
  _cpanel->setOrigSRate(_urec->getSamplingRate());
  _cpanel->getCurSP()->setNum(_urec->getSamplingRate());

  _cpanel->updateCurSRate(_cpanel->getSPLCD()->value());
  _cpanel->updateLeadTime(_cpanel->getLeadLCD()->value());
  _cpanel->updateNoiseD(_cpanel->getNoiseLCD()->value());
  
}


void
GlobalWidget::saveLogFile(QString filename)
{
  QString text = _textbox->text();
  QFile f( filename );
  if ( !f.open( IO_WriteOnly ) ) {
    _status->message( QString("Could not write to %1").arg(filename),
			  2000 );
    return;
  }
  
  QTextStream t( &f );
  t << text;
  f.close();
  
  _status->message("Log Saved.", 4000);

}


void
GlobalWidget::sendUserDataToMatlab()
{
  _urec->sendToMatlab(_mengine);
}



/////////////////////////////////////////////////////////

// definitions for the Control Panel widget

ControlPanel::ControlPanel(QWidget *parent, const char *name) : QWidget(parent, name)
{
  
  // create the labels

  _filelabel = new QLabel("Data Set: ", this, "Data_set");
  _curfile   = new QLabel("None", this, "CurFile");
  _curfile->setIndent(TRUE);
  _predlabel = new QLabel("Predictor: ", this, "predictor");
  _curpred   = new QLabel("None", this, "CurPred");
  _curpred->setIndent(TRUE);
   
  _sprate = new QLabel("Original SR(Hz): ", this, "srate");
  _origsp  = new QLabel(this, "cursrate");
  _origsp->setNum(0.0);
  _origsp->setIndent(TRUE);

  _newsprate = new QLabel("Current SR(Hz): ", this, "srate2");
  _cursp = new QLabel(this, "cursrate2");
  _cursp->setNum(0.0);
  _cursp->setIndent(TRUE); 

  _predint = new QLabel("Lead Time(ms): ", this, "leadtime");
  _curpredint = new QLabel(this, "curpredint");
  _curpredint->setNum(0.0);
  _curpredint->setIndent(TRUE); 
  

  _errormet = new QLabel("Error Metric: ", this, "errormetric");
  _curerr = new QLabel("None", this, "curerr");
  _curerr->setIndent(TRUE); 

  // create the sampling rate adjuster
  _splcd = new QLCDNumber(2, this, "splcd");
  _splcd->setSegmentStyle(QLCDNumber::Flat);
  QSlider *spslider = new QSlider(Horizontal, this, "spslider");
  spslider->setRange(1,40);
  spslider->setValue(1);
  _splcd->display(1);
  QLabel *sradjust = new QLabel("Sampling Rate Adjuster", this, "sladjuster");
  _cursrate = 0.0;
  _origsrate = 0.0;
  _srateint = 0;

  connect(spslider, SIGNAL(valueChanged(int)), _splcd, SLOT(display(int)) );
  connect(spslider, SIGNAL(valueChanged(int)), this, SLOT(updateCurSRate(int)));

  // create the lead time adjuster
  _leadt = new QLCDNumber(3, this, "leadt");
  _leadt->setSegmentStyle(QLCDNumber::Flat);
  QSlider *leadslider = new QSlider(Horizontal, this, "leadslider");
  leadslider->setRange(16,250);
  leadslider->setValue(16);
  _leadt->display(16);
  QLabel *leadadj = new QLabel("Lead Time Adjuster(units)", this, "leadadjuster");
  _leadtime = 0.0;
  _leadtimeint = 0;

  connect(leadslider, SIGNAL(valueChanged(int)), _leadt, SLOT(display(int)) );
  connect(leadslider, SIGNAL(valueChanged(int)), this, SLOT(updateLeadTime(int)));
  
  // create the noise variance adjuster
  _noisev = new QLCDNumberD(this, "noisev");
  _noisev->setNumDigits(12);
  _noisev->setSegmentStyle(QLCDNumber::Flat);
  QSlider *noiseslider = new QSlider(Horizontal, this, "noiseslider");
  noiseslider->setRange(0,100);
  noiseslider->setValue(0);
  _noisev->display(0.0);
  QLabel *noiseadj = new QLabel("Noise Variance Adjuster", this, "noiseadjuster");
  _noisevar = 0.0;

  connect(noiseslider, SIGNAL(valueChanged(int)), _noisev, SLOT(myDisplay(int)) );
  connect(noiseslider, SIGNAL(valueChanged(int)), this, SLOT(updateNoise(int)));
 
  QGridLayout *layout1 = new QGridLayout(this, 8,8,1,0,"Layout1");
  layout1->setSpacing(5);
  layout1->addWidget(_filelabel, 0,0);
  layout1->addWidget(_curfile, 0,1);
  layout1->addWidget(_sprate,1,0);
  layout1->addWidget(_origsp, 1,1);
  layout1->addWidget(_newsprate,2,0);
  layout1->addWidget(_cursp,2,1);
  layout1->addWidget(_predlabel,3,0);
  layout1->addWidget(_curpred, 3,1);
  layout1->addWidget(_predint,4,0);
  layout1->addWidget(_curpredint, 4,1);
  layout1->addWidget(_splcd, 0,3);
  layout1->addWidget(spslider,1,3);
  layout1->addWidget(sradjust, 2,3);
  layout1->addWidget(_leadt, 0,4);
  layout1->addWidget(leadslider,1,4);
  layout1->addWidget(leadadj, 2,4);
  layout1->addWidget(_noisev, 0,5);
  layout1->addWidget(noiseslider,1,5);
  layout1->addWidget(noiseadj, 2,5);
  layout1->addWidget(_errormet, 5,0);
  layout1->addWidget(_curerr, 5,1);
 
}

// destructor
ControlPanel::~ControlPanel()
{
  delete _filelabel;
  delete _curfile;
  delete _predlabel;
  delete _curpred;
  delete _sprate;
  delete _origsp;
  delete _newsprate;
  delete _cursp;
  delete _predint;
  delete _curpredint;
  delete _errormet;
  delete _curerr;
  
  delete _splcd;
  delete _leadt;
  delete _noisev;  
}


/////////////////////////////////////////////
// Slots for Control Panel

void
ControlPanel::updateNoise(int num)
{
 if (num == 0)
   _noisevar = 0.0;
  else
   _noisevar = ((double)num/NOISED);
}


void
ControlPanel::updateNoiseD(double num)
{
 _noisevar = num;
}

void 
ControlPanel::updateCurSRate(int num)
{ 
  _srateint = num;
  _cursrate = _origsrate / (double)num;
  _cursp->setNum(_cursrate);
}


void 
ControlPanel::updateLeadTime(int num)
{
 if (_origsrate == 0)
  return;
 _leadtimeint = num;
 _leadtime = 1000.0 * num * (1.0 / _origsrate);
 _curpredint->setNum(_leadtime);

}

/////////////////////////////////////
// member functions for the Matlab Engine class
// these routines act a wrapper around the engine's C api


int 
MATLABEngine::startEngine()
{
  if (!(_eng = engOpen("\0"))) 
    return 0;
  
// we were successful
  return 1;

}

void
MATLABEngine::killEngine()
{
  // kill engine as long as we have created it
  if (_eng)
    engClose(_eng); 

}

int
MATLABEngine::sendArray(const mxArray *mp)
{
  if (_eng) {
   int v = engPutArray(_eng, mp);
   if (v == 0)
     return 1;
   else
     return 0;
  }
  return 0;

}

mxArray *
MATLABEngine::getArray(const char *name)
{
  if (_eng) 
    return engGetArray(_eng, name);
  else
    return 0;
}


int
MATLABEngine::runCmd(QString cmd)
{
  if (_eng) {
    int v = engEvalString(_eng, cmd.latin1());
    if (v == 0)
      return 1;
    else 
      return 0;
  }
  else
    return 0;
}

///////////////////////////////////////////
// member functions for dialogs
// functions for plots dialog

PlotsDialog::PlotsDialog(QWidget *parent, const char *name) : QWidget(parent, name),
                                                              _showx(0),  _showy(0),
                                                              _showz(0),  _showqx(0),
                                                              _showqy(0), _showqz(0),
                                                              _showqw(0)
{
  // create the close buttons 
  this->setGeometry(0,0,200,150);
  QVBoxLayout *vbox = new QVBoxLayout( this );
  QHBoxLayout *box1 = new QHBoxLayout( vbox );

  _closebtn = new QPushButton("Close", this, "closeButton");
  connect(_closebtn, SIGNAL( clicked() ), this, SLOT( hideDialog() ));  

  QButtonGroup *posgrp = new QButtonGroup(1, QGroupBox::Horizontal, "Position",
                                          this,"posgrp");

  QCheckBox *cx = new QCheckBox("X", posgrp);
  QCheckBox *cy = new QCheckBox("Y", posgrp);
  QCheckBox *cz = new QCheckBox("Z", posgrp);
  
  connect(cx, SIGNAL( toggled(bool) ), this, SLOT( toggleX(bool) ));
  connect(cy, SIGNAL( toggled(bool) ), this, SLOT( toggleY(bool) ));
  connect(cz, SIGNAL( toggled(bool) ), this, SLOT( toggleZ(bool) ));
 
  QButtonGroup *rotgrp = new QButtonGroup(1, QGroupBox::Horizontal, "Orientation",
                                          this,"rotgrp");

  QCheckBox *cqx = new QCheckBox("QX", rotgrp);
  QCheckBox *cqy = new QCheckBox("QY", rotgrp);
  QCheckBox *cqz = new QCheckBox("QZ", rotgrp);
  QCheckBox *cqw = new QCheckBox("QW", rotgrp);

  connect(cqx, SIGNAL( toggled(bool) ), this, SLOT( toggleQX(bool) ));
  connect(cqy, SIGNAL( toggled(bool) ), this, SLOT( toggleQY(bool) ));
  connect(cqz, SIGNAL( toggled(bool) ), this, SLOT( toggleQZ(bool) ));
  connect(cqw, SIGNAL( toggled(bool) ), this, SLOT( toggleQW(bool) ));  

  box1->addWidget(posgrp);
  box1->addWidget(rotgrp);
  vbox->addWidget(_closebtn);
  this->setCaption("Plot Selector");
}

// member functions for the local polynomial dialog

LocalPolyDialog::LocalPolyDialog(QWidget *parent, const char *name) : 
                                                               QWidget(parent, name),
                                                               _pdegree(0), _pwindow(1),
                                                               _derfly(0), _kf(0)
{
  // create the close buttons 
  _polytype = tNone3;

  this->setGeometry(0,0,220,240);
  QVBoxLayout *vbox = new QVBoxLayout( this );
  QHBoxLayout *box1 = new QHBoxLayout( vbox );

  _closebtn = new QPushButton("Close", this, "closeButton");
  connect(_closebtn, SIGNAL( clicked() ), this, SLOT( hideDialog() ));  

  QButtonGroup *alggrp = new QButtonGroup(1, QGroupBox::Horizontal, "Algorithm",
                                          this,"alggrp");
 
  QRadioButton *lang    = new QRadioButton("Lagrangian", alggrp);
  QRadioButton *ls      = new QRadioButton("Least Squares", alggrp);
  QRadioButton *cspline = new QRadioButton("Cubic Spline", alggrp);
  QRadioButton *taylor  = new QRadioButton("2nd Order Taylor", alggrp);

  connect(lang,    SIGNAL( clicked() ), this, SLOT( setLagrange() ));
  connect(ls,      SIGNAL( clicked() ), this, SLOT( setLS() ));
  connect(cspline, SIGNAL( clicked() ), this, SLOT( setCSpline() ));
  connect(taylor,  SIGNAL( clicked() ), this, SLOT( setTaylor() ));  

  _window = new QLCDNumber(2, this, "window");
  _window->setSegmentStyle(QLCDNumber::Flat);
  QSlider *winslider = new QSlider(Horizontal, this, "winslider");
  winslider->setRange(1,50);
  winslider->setValue(1);
  _window->display(1);
  QLabel *winlabel = new QLabel("Pose Window Size:", this, "winlabel");  
  connect(winslider, SIGNAL(valueChanged(int)), _window, SLOT(display(int)) ); 
  connect(winslider, SIGNAL(valueChanged(int)), this, SLOT(setWindow(int)) );

  _deg = new QLCDNumber(2, this, "degree");
  _deg->setSegmentStyle(QLCDNumber::Flat);
  _degslider = new QSlider(Horizontal, this, "degslider");
  _degslider->setRange(0,25);
  _degslider->setValue(0);
  _deg->display(0);
  _deglabel = new QLabel("Polynomial Degree:", this, "deglabel");  
  connect(_degslider, SIGNAL(valueChanged(int)), _deg, SLOT(display(int)) ); 
  connect(_degslider, SIGNAL(valueChanged(int)), this, SLOT(setDegree(int)) );

  QCheckBox *fly = new QCheckBox("Real-Time Derivatives", this);
  connect(fly, SIGNAL( toggled(bool) ), this, SLOT( toggleFly(bool) ));

  QCheckBox *kf = new QCheckBox("Use Kalman Filtering", this);
  connect(kf, SIGNAL( toggled(bool) ), this, SLOT( toggleKF(bool) ));

  box1->addWidget(alggrp);
  vbox->addWidget(winlabel);
  vbox->addWidget(_window);
  vbox->addWidget(winslider);
  vbox->addWidget(_deglabel);
  vbox->addWidget(_deg);
  vbox->addWidget(_degslider);
  vbox->addWidget(fly);
  vbox->addWidget(kf);
  vbox->addWidget(_closebtn);
  this->setCaption("Local Polynomial Params");
}


// member functions for the exponential smoothing dialog


ExpSmoothDialog::ExpSmoothDialog(QWidget *parent, const char *name) : 
                                                              QWidget(parent, name),
                                                              _alpha(0.01), _beta(0.01),
                                                              _kf(0)
{
  // create the close buttons 
  _expsmtype = tNone4;

  this->setGeometry(0,0,240,200);
  QVBoxLayout *vbox = new QVBoxLayout( this );
  QHBoxLayout *box1 = new QHBoxLayout( vbox );

  _closebtn = new QPushButton("Close", this, "closeButton");
  connect(_closebtn, SIGNAL( clicked() ), this, SLOT( hideDialog() ));  

  QButtonGroup *alggrp = new QButtonGroup(1, QGroupBox::Horizontal, "Algorithm",
                                          this,"alggrp");
 
  QRadioButton *s      = new QRadioButton("Simple Exponential", alggrp);
  QRadioButton *d      = new QRadioButton("Double Exponential", alggrp);
  QRadioButton *hw = new QRadioButton("Holt-Winters Exponential", alggrp);

  connect(s,      SIGNAL( clicked() ), this, SLOT( setSimpleExp() ));
  connect(d,      SIGNAL( clicked() ), this, SLOT( setDoubleExp() ));
  connect(hw, SIGNAL( clicked() ), this, SLOT( setHoltWinters() )); 
 
  QLCDNumberD *a = new QLCDNumberD(this, "a");
  a->setSegmentStyle(QLCDNumber::Flat);
  a->setNumDigits(4);
  QSlider *aslider = new QSlider(Horizontal, this, "aslider");
  aslider->setRange(1,100);
  aslider->setValue(1);
  a->display(0.01);
  QLabel *alabel = new QLabel("Global Alpha:", this, "alabel");  
  connect(aslider, SIGNAL(valueChanged(int)), a, SLOT(myDisplay2(int)) ); 
  connect(aslider, SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)) );

  QLCDNumberD *b = new QLCDNumberD(this, "b");
  b->setSegmentStyle(QLCDNumber::Flat);
  b->setNumDigits(4);
  QSlider *bslider = new QSlider(Horizontal, this, "bslider");
  bslider->setRange(1,100);
  bslider->setValue(1);
  b->display(0.01);
  QLabel *blabel = new QLabel("Global Beta:", this, "alabel");  
  connect(bslider, SIGNAL(valueChanged(int)), b, SLOT(myDisplay2(int)) ); 
  connect(bslider, SIGNAL(valueChanged(int)), this, SLOT(setBeta(int)) );

  QCheckBox *kf = new QCheckBox("Use Kalman Filtering", this);
  connect(kf, SIGNAL( toggled(bool) ), this, SLOT( toggleKF(bool) )); 

  box1->addWidget(alggrp);
  vbox->addWidget(alabel);
  vbox->addWidget(a);
  vbox->addWidget(aslider);
  vbox->addWidget(blabel);
  vbox->addWidget(b);
  vbox->addWidget(bslider);
  vbox->addWidget(kf);
  vbox->addWidget(_closebtn);
  this->setCaption("Exponential Smoothing Params");
}


void
ExpSmoothDialog::setAlpha(int num)
{
  // turn num into a double
  if (num == 0)
   _alpha = 0.01; 
  else
   _alpha = (double)num/EXPPARAM;
}

void
ExpSmoothDialog::setBeta(int num)
{
  // turn num into a double
  if (num == 0)
   _beta = 0.01; 
  else
   _beta = (double)num/EXPPARAM;
}


// member functions for grey Pred dialog
GreyPredDialog::GreyPredDialog(QWidget *parent, const char *name) : 
                                                              QWidget(parent, name),
                                                              _numpts(4), _kf(0)
{
  this->setGeometry(0,0,240,100);
  QVBoxLayout *vbox = new QVBoxLayout( this );

  _closebtn = new QPushButton("Close", this, "closeButton");
  connect(_closebtn, SIGNAL( clicked() ), this, SLOT( hideDialog() ));  

   QLCDNumber *pts= new QLCDNumber(2, this, "numpts");
   pts->setSegmentStyle(QLCDNumber::Flat);
  QSlider *ptsslider = new QSlider(Horizontal, this, "ptsslider");
  ptsslider->setRange(4,20);
  ptsslider->setValue(4);
  pts->display(4);
  QLabel *label = new QLabel("Number of Points:", this, "ptslabel");  
  connect(ptsslider, SIGNAL(valueChanged(int)), pts, SLOT(display(int)) ); 
  connect(ptsslider, SIGNAL(valueChanged(int)), this, SLOT(setNumPts(int)) );

  QCheckBox *kf = new QCheckBox("Use Kalman Filtering", this);
  connect(kf, SIGNAL( toggled(bool) ), this, SLOT( toggleKF(bool) )); 

  vbox->addWidget(label);
  vbox->addWidget(pts);
  vbox->addWidget(ptsslider);
  vbox->addWidget(kf);
  vbox->addWidget(_closebtn);
  this->setCaption("Grey System Params");
}  



// member functions for the Kalman Filter Predictor dialog
KalmanPredDialog:: KalmanPredDialog(QWidget *parent, const char *name) :
                                                             QWidget(parent, name),
                                                             _specd(0), _ukfon(0)
{
   _ktype = tNone5;

  this->setGeometry(0,0,260,300);
  QVBoxLayout *vbox = new QVBoxLayout( this );
  QHBoxLayout *box1 = new QHBoxLayout( vbox );

  _closebtn = new QPushButton("Close", this, "closeButton");
  connect(_closebtn, SIGNAL( clicked() ), this, SLOT( hideDialog() ));  

  QButtonGroup *alggrp = new QButtonGroup(1, QGroupBox::Horizontal, "Algorithm",
                                          this,"alggrp");
 
  QRadioButton *k1 = new QRadioButton("Pos/Vel, Single Measure", alggrp);
  QRadioButton *k2 = new QRadioButton("Pos/Vel/Acc, Single Measure", alggrp);  
  QRadioButton *k3 = new QRadioButton("Pos/Vel, Multiple Measure (PV)", alggrp);
  QRadioButton *k4 = new QRadioButton("Pos/Vec/Acc, Multiple Measure (PV)", alggrp);
  QRadioButton *k5 = new QRadioButton("Pos/Vec/Acc, Multiple Measure (PA)", alggrp);
  QRadioButton *k6 = new QRadioButton("Pos/Vec/Acc, Multiple Measure (PVA)", alggrp);
  
  connect(k1, SIGNAL( clicked() ), this, SLOT( setPVSM() ));
  connect(k2, SIGNAL( clicked() ), this, SLOT( setPVASM() ));
  connect(k3, SIGNAL( clicked() ), this, SLOT( setPVMMV() ));
  connect(k4, SIGNAL( clicked() ), this, SLOT( setPVAMMV() ));
  connect(k5, SIGNAL( clicked() ), this, SLOT( setPVAMMA() ));
  connect(k6, SIGNAL( clicked() ), this, SLOT( setPVAMMVA() ));
  

  QLCDNumber *sd = new QLCDNumber(3, this, "specd");
  sd->setSegmentStyle(QLCDNumber::Flat);
  QSlider *ptsslider = new QSlider(Horizontal, this, "spdslider");
  ptsslider->setRange(0,250);
  ptsslider->setValue(0);
  sd->display(0);
  QLabel *label = new QLabel("Spectral Denstity Factor: ", this, "ptslabel");  
  connect(ptsslider, SIGNAL(valueChanged(int)), sd, SLOT(display(int)) ); 
  connect(ptsslider, SIGNAL(valueChanged(int)), this, SLOT(setSpecD(int)) );

  QCheckBox *ukf = new QCheckBox("Use Unscented Kalman Filter", this);
  connect(ukf, SIGNAL( toggled(bool) ), this, SLOT( toggleUKF(bool) ));

  box1->addWidget(alggrp);
  vbox->addWidget(label);
  vbox->addWidget(sd);
  vbox->addWidget(ptsslider);
  vbox->addWidget(ukf);
  vbox->addWidget(_closebtn);
  this->setCaption("Kalman Filter-Based Predictors");
}  


// member functions for the CI predictor dialog

CIPredDialog::CIPredDialog(QWidget *parent, const char *name) :
                                                             QWidget(parent, name),
                                                             _specd(0)
{

   _citype = tNone6;
   
   this->setGeometry(0,0,260,200);
   QVBoxLayout *vbox = new QVBoxLayout( this );
   QHBoxLayout *box1 = new QHBoxLayout( vbox );

   _closebtn = new QPushButton("Close", this, "closeButton");
   connect(_closebtn, SIGNAL( clicked() ), this, SLOT( hideDialog() ));  

   QButtonGroup *alggrp = new QButtonGroup(1, QGroupBox::Horizontal, "Algorithm",
                                          this,"alggrp");
 
   QRadioButton *c1 = new QRadioButton("PVSM CI", alggrp);
   QRadioButton *c2 = new QRadioButton("PVMMV CI", alggrp);
   
   connect(c1, SIGNAL( clicked() ), this, SLOT( setCIPVSM() ));
   connect(c2, SIGNAL( clicked() ), this, SLOT( setCIPVMMV() ));

   QLCDNumber *sd = new QLCDNumber(3, this, "cispecd");
   sd->setSegmentStyle(QLCDNumber::Flat);
   QSlider *ptsslider = new QSlider(Horizontal, this, "cispdslider");
   ptsslider->setRange(0,250);
   ptsslider->setValue(0);
   sd->display(0);
   QLabel *label = new QLabel("Spectral Denstity Factor: ", this, "ciptslabel");  
   connect(ptsslider, SIGNAL(valueChanged(int)), sd, SLOT(display(int)) ); 
   connect(ptsslider, SIGNAL(valueChanged(int)), this, SLOT(setCISpecD(int)) );

   box1->addWidget(alggrp);
   vbox->addWidget(label);
   vbox->addWidget(sd);
   vbox->addWidget(ptsslider);
   vbox->addWidget(_closebtn);
   this->setCaption("Cov. Intersection-Based Predictors");
}


// member functions for the MMAE predictor dialog

MMAEPredDialog::MMAEPredDialog(QWidget *parent, const char *name) : 
                                                                 QWidget(parent,name),
                                                                 _specd(0),
                                                                 _seperator(0)
{
  _mmaetype = tNone7;
  _combiner = tSUM;

  this->setGeometry(0,0,300,220);
  
  QVBoxLayout *vbox = new QVBoxLayout( this );
  QHBoxLayout *box1 = new QHBoxLayout( vbox );

  _closebtn = new QPushButton("Close", this, "closeButton");
  connect(_closebtn, SIGNAL( clicked() ), this, SLOT( hideDialog() ));  

  QButtonGroup *alggrp = new QButtonGroup(1, QGroupBox::Horizontal, "Algorithm",
                                          this,"alggrp");
 
  QRadioButton *m1 = new QRadioButton("3 Model MMAE", alggrp);
  QRadioButton *m2 = new QRadioButton("3 Model MMAE Component Wise (Pos Only)", alggrp);
  QRadioButton *m3 = new QRadioButton("Multiple Param, Single Model MMAE", alggrp);     

  connect(m1, SIGNAL( clicked() ), this, SLOT( setMMAE3Mod() ));
  connect(m2, SIGNAL( clicked() ), this, SLOT( setMMAE3ModComp() ));
  connect(m3, SIGNAL( clicked() ), this, SLOT( setMMAEMulParam() ));  
 
  QLCDNumber *sd = new QLCDNumber(3, this, "mmaespecd");
  sd->setSegmentStyle(QLCDNumber::Flat);
  QSlider *ptsslider = new QSlider(Horizontal, this, "mmaespdslider");
  ptsslider->setRange(0,250);
  ptsslider->setValue(0);
  sd->display(0);
  QLabel *label = new QLabel("Spectral Density Factor: ", this, "mmseptslabel");  
  connect(ptsslider, SIGNAL(valueChanged(int)), sd, SLOT(display(int)) ); 
  connect(ptsslider, SIGNAL(valueChanged(int)), this, SLOT(setMMAESpecD(int)) );
  
  QLCDNumber *sep = new QLCDNumber(3, this, "Seperator");
  sep->setSegmentStyle(QLCDNumber::Flat);
  QSlider *sepslider = new QSlider(Horizontal, this, "mmaesepslider");
  sepslider->setRange(0,250);
  sepslider->setValue(0);
  sep->display(0);
  QLabel *label2 = new QLabel("Spectral Density Seperator: ", this, "seplabel");  
  connect(sepslider, SIGNAL(valueChanged(int)), sep, SLOT(display(int)) ); 
  connect(sepslider, SIGNAL(valueChanged(int)), this, SLOT(setMMAESpecDSep(int)) );


  QButtonGroup *cmbgrp = new QButtonGroup(1, QGroupBox::Horizontal, "Combiner",
                                          this, "cmbgrp");
  QRadioButton *c1 = new QRadioButton("Weighted Sum", cmbgrp);
  QRadioButton *c2 = new QRadioButton("Max Probability", cmbgrp);

  connect(c1, SIGNAL( clicked() ), this, SLOT( setSUM() ));
  connect(c2, SIGNAL( clicked() ), this, SLOT( setMAX() ));

   box1->addWidget(alggrp);
   vbox->addWidget(label);
   vbox->addWidget(sd);
   vbox->addWidget(ptsslider);
   vbox->addWidget(label2);
   vbox->addWidget(sep);
   vbox->addWidget(sepslider);
   vbox->addWidget(cmbgrp);
   vbox->addWidget(_closebtn);
   this->setCaption("MMAE-Based Predictors");

}

////////////////////////////////////////////////////////

// member functions for the signal analysis dialog

SignalAnalysisDialog::SignalAnalysisDialog(GlobalWidget *gw, QWidget *parent,
                                           const char *name) :
                                                               QWidget(parent,name),
                                                               _lag(0),
                                                               _gbw(gw)
                                                                
{
  _analyzer = new SignalAnalyzer(gw);
  _atype = tNone8;

  this->setGeometry(0,0,330,220);
  
  QVBoxLayout *vbox = new QVBoxLayout( this );
  QHBoxLayout *box1 = new QHBoxLayout( vbox );

  _closebtn = new QPushButton("Close", this, "closeButton");
  connect(_closebtn, SIGNAL( clicked() ), this, SLOT( hideDialog() ));  
  _execbtn = new QPushButton("Execute", this, "execButton");
  connect(_execbtn, SIGNAL( clicked() ), this, SLOT( exec() ));

  QButtonGroup *alggrp = new QButtonGroup(1, QGroupBox::Horizontal, "Analysis Tool",
                                          this,"analysisgrp");

  QRadioButton *m3 = new QRadioButton("Basic Signal Metrics", alggrp);
  QRadioButton *m1 = new QRadioButton("Sample Autocorrelation Function (SACF)", alggrp);
  QRadioButton *m2 = new QRadioButton("Window-Based SACF Computation", alggrp);
  QRadioButton *m4 = new QRadioButton("Power Spectral Density", alggrp);
  QRadioButton *m5 = new QRadioButton("CPPDP Metric", alggrp);
  QRadioButton *m6 = new QRadioButton("Basic Velocity-Based Metrics", alggrp); 


  connect(m1, SIGNAL( clicked() ), this, SLOT( setSACF() ));
  connect(m2, SIGNAL( clicked() ), this, SLOT( setACFWindowD() ));
  connect(m3, SIGNAL( clicked() ), this, SLOT( setBasicSMetrics() ));
  connect(m4, SIGNAL( clicked() ), this, SLOT( setPSD() ));
  connect(m5, SIGNAL( clicked() ), this, SLOT( setCrPt() ));
  connect(m6, SIGNAL( clicked() ), this, SLOT( setVelBasedMetrics() ));


  QLCDNumberD *lagsd = new QLCDNumberD(this, "lagsd");
  lagsd->setSegmentStyle(QLCDNumber::Flat);
  lagsd->setNumDigits(4);
  QSlider *lagslider = new QSlider(Horizontal, this, "lagslider");
  lagslider->setRange(0,500);
  lagslider->setValue(0);
  lagsd->display(0);
  QLabel *laglabel = new QLabel("Lag: ", this, "laglabel");  
  connect(lagslider, SIGNAL(valueChanged(int)), lagsd, SLOT(myDisplay3(int)) ); 
  connect(lagslider, SIGNAL(valueChanged(int)), this, SLOT(setLagValue(int)) );

  QLCDNumber *cut = new QLCDNumberD(this, "cut");
  cut->setSegmentStyle(QLCDNumber::Flat);
  cut->setNumDigits(3);
  QSlider *cutslider = new QSlider(Horizontal, this, "cutslider");
  cutslider->setRange(1,500);
  cutslider->setValue(1);
  cut->display(1);
  QLabel *cutlabel = new QLabel("Window Size: ", this, "cutlabel");  
  connect(cutslider, SIGNAL(valueChanged(int)), cut, SLOT(display(int)) ); 
  connect(cutslider, SIGNAL(valueChanged(int)), this, SLOT(setCutOff(int)) ); 


  box1->addWidget(alggrp);
  vbox->addWidget(laglabel);
  vbox->addWidget(lagsd);
  vbox->addWidget(lagslider);
  vbox->addWidget(cutlabel);
  vbox->addWidget(cut);
  vbox->addWidget(cutslider);
  QHBoxLayout *box2 = new QHBoxLayout( vbox );
  box2->addWidget(_execbtn);
  box2->addWidget(_closebtn); 
  this->setCaption("Signal Analysis and Quantification");

}

// destructor
SignalAnalysisDialog::~SignalAnalysisDialog()
{ 
   delete _analyzer;
   delete _closebtn;
   delete _execbtn;
}


// the execution routine
void
SignalAnalysisDialog::exec()
{
   // make sure we have a data set
   if (!_gbw->getDataPresent()) {
       _gbw->getStatusBar()->message("No data set loaded.", 5000); 
       return;
   }
   
    QApplication::setOverrideCursor(Qt::waitCursor);
   // switch on the analyzer type
   switch(_atype) {
     case tSACF:
        _analyzer->prepare();
        _analyzer->computeSACF(_lag);
        _analyzer->displaySACFResults();
        _gbw->getStatusBar()->message("Sample ACF Computed.", 8000);
        break;
     case tACFWin:
        _analyzer->prepare();
        _analyzer->computeACFWindows(_initwin);
        _analyzer->displayWinACFResults();
	_gbw->getStatusBar()->message("ACF Windows Computed.", 8000);
        break;
     case tBasic:
       _analyzer->prepare();
       _analyzer->computeBasicSignalMetrics();
       _analyzer->displayBasicSignalMetrics();
       break;
     case tPSD:
       _analyzer->prepare();
       _analyzer->computeAndDisplayPSD();
       break;
     case tCrPt: 
       _analyzer->prepare();
       _analyzer->computeAndDisplayCrPt();
       break;
     case tVelBasic:
       _analyzer->prepare();
       _analyzer->computeBasicVelMetrics();
       _analyzer->displayBasicVelMetrics();
       break;
     default:
      _gbw->getStatusBar()->message("No Analysis Tool Selected.", 8000);
   }
   QApplication::restoreOverrideCursor();
}

//////////////////////////////////////////////////////
// Member functions for the numerical differentation tools dialog

NDiffToolDialog::NDiffToolDialog(GlobalWidget *gw, QWidget *parent,
                                 const char *name) :
                                                    QWidget(parent,name),
                                                    _gbw(gw), _deg(1), _win(1),
                                                    _left(0), _right(0),
                                                    _cdiff(0), _hs(0),
                                                    _avgdiff(0)
						                                                                    
{
  _ndifftools = new NumDiff(gw);
  _ntype = tNone9;
  _threshtype = tNone10;  

  this->setGeometry(0,0,330,320);
  
  QVBoxLayout *vbox = new QVBoxLayout( this );
  QHBoxLayout *box1 = new QHBoxLayout( vbox );
  QHBoxLayout *box3 = new QHBoxLayout( vbox );
  QHBoxLayout *box4 = new QHBoxLayout( vbox );

  _closebtn = new QPushButton("Close", this, "closeButton");
  connect(_closebtn, SIGNAL( clicked() ), this, SLOT( hideDialog() ));  
  _execbtn = new QPushButton("Execute", this, "execButton");
  connect(_execbtn, SIGNAL( clicked() ), this, SLOT( exec() ));

  QButtonGroup *alggrp = new QButtonGroup(1, QGroupBox::Horizontal, 
                                          "Num Diff Tools",
                                          this,"ndiffgrp");

  QRadioButton *m1 = new QRadioButton("Simple Numerical Diff", alggrp);
  QRadioButton *m2 = new QRadioButton("Orthogonal Least Squares", alggrp);
  QRadioButton *m3 = new QRadioButton("Savitsky-Golay Smoothing", alggrp);
  QRadioButton *m4 = new QRadioButton("Wavelet Denoising - Daubechies 4", alggrp);
  QRadioButton *m5 = new QRadioButton("Wavelet Denoising - Daubechies 8", alggrp);
  QRadioButton *m6 = new QRadioButton("Wavelet Denoising - Symlet 8", alggrp);
  QRadioButton *m7 = new QRadioButton("Method of Mollification", alggrp);
  QRadioButton *m8 = new QRadioButton("Anderssen-Bloomfield Regularization (Odd)", alggrp);
  QRadioButton *m9 = new QRadioButton("Anderssen-Bloomfield Regularization (No Odd)", alggrp);
  QRadioButton *m10 = new QRadioButton("Kosarev-Pantos Window (Odd)", alggrp);
  QRadioButton *m11 = new QRadioButton("Kosarev-Pantos Window (No Odd)", alggrp);

  connect(m1, SIGNAL( clicked() ), this, SLOT( setSimpleND() ));
  connect(m2, SIGNAL( clicked() ), this, SLOT( setLSOrtho() ));
  connect(m3, SIGNAL( clicked() ), this, SLOT( setSavitskyGolay() ));
  connect(m4, SIGNAL( clicked() ), this, SLOT( setWaveletDB4() ));
  connect(m5, SIGNAL( clicked() ), this, SLOT( setWaveletDB8() ));  
  connect(m6, SIGNAL( clicked() ), this, SLOT( setWaveletSYM8() ));  
  connect(m7, SIGNAL( clicked() ), this, SLOT( setMollify() ));
  connect(m8, SIGNAL( clicked() ), this, SLOT( setABOdd() ));
  connect(m9, SIGNAL( clicked() ), this, SLOT( setABNoOdd() ));
  connect(m10, SIGNAL( clicked() ), this, SLOT( setKPOdd() ));
  connect(m11, SIGNAL( clicked() ), this, SLOT( setKPNoOdd() ));  


  QButtonGroup *threshgrp = new QButtonGroup(1, QGroupBox::Horizontal, 
                                          "Thresholding Method",
                                          this,"threshgrp"); 

  QRadioButton *t1 = new QRadioButton("Stein's Unbiased Risk", threshgrp);
  QRadioButton *t2 = new QRadioButton("Heuristic SUR", threshgrp);
  QRadioButton *t3 = new QRadioButton("Universal", threshgrp);
  QRadioButton *t4 = new QRadioButton("Minimax", threshgrp);  
  QRadioButton *t5 = new QRadioButton("Kill Details", threshgrp);

  connect(t1, SIGNAL( clicked() ), this, SLOT( setRigrSure() ));
  connect(t2, SIGNAL( clicked() ), this, SLOT( setHeurSure() ));
  connect(t3, SIGNAL( clicked() ), this, SLOT( setSqtwoLog() ));
  connect(t4, SIGNAL( clicked() ), this, SLOT( setMinimax() ));
  connect(t5, SIGNAL( clicked() ), this, SLOT( setKillDet() ));

  QButtonGroup *hsgrp = new QButtonGroup(1, QGroupBox::Horizontal, 
                                          "Thresholding Style",this,"hsgrp"); 
  
  QRadioButton *hs1 = new QRadioButton("Hard", hsgrp);
  QRadioButton *hs2 = new QRadioButton("Soft", hsgrp); 
  
  connect(hs1, SIGNAL( clicked() ), this, SLOT( setHardThresh() ));
  connect(hs2, SIGNAL( clicked() ), this, SLOT( setSoftThresh() ));

  QLCDNumber *win = new QLCDNumber(this, "winsize");
  win->setSegmentStyle(QLCDNumber::Flat);
  win->setNumDigits(3);
  QSlider *winslider = new QSlider(Horizontal, this, "winslider");
  winslider->setRange(1,200);
  winslider->setValue(1);
  win->display(1);
  QLabel *winlabel = new QLabel("Window Size: ", this, "winlabel");  
  connect(winslider, SIGNAL(valueChanged(int)), win, SLOT(display(int)) ); 
  connect(winslider, SIGNAL(valueChanged(int)), this, SLOT(setWin(int)) );

  QLCDNumber *deg = new QLCDNumber(this, "degree");
  deg->setSegmentStyle(QLCDNumber::Flat);
  deg->setNumDigits(2);
  QSlider *degslider = new QSlider(Horizontal, this, "degslider");
  degslider->setRange(1,10);
  degslider->setValue(1);
  deg->display(1);
  QLabel *deglabel = new QLabel("Degree: ", this, "deglabel");  
  connect(degslider, SIGNAL(valueChanged(int)), deg, SLOT(display(int)) ); 
  connect(degslider, SIGNAL(valueChanged(int)), this, SLOT(setDeg(int)) );

  QLCDNumber *left = new QLCDNumber(this, "left");
  left->setSegmentStyle(QLCDNumber::Flat);
  left->setNumDigits(3);
  QSlider *leftslider = new QSlider(Horizontal, this, "leftslider");
  leftslider->setRange(0,128);
  leftslider->setValue(0);
  left->display(0);
  QLabel *leftlabel = new QLabel("Left: ", this, "leftlabel");  
  connect(leftslider, SIGNAL(valueChanged(int)), left, SLOT(display(int)) ); 
  connect(leftslider, SIGNAL(valueChanged(int)), this, SLOT(setLeft(int)) );

  QLCDNumber *right = new QLCDNumber(this, "right");
  right->setSegmentStyle(QLCDNumber::Flat);
  right->setNumDigits(3);
  QSlider *rightslider = new QSlider(Horizontal, this, "rightslider");
  rightslider->setRange(0,128);
  rightslider->setValue(0);
  right->display(0);
  QLabel *rightlabel = new QLabel("Right: ", this, "rightlabel");  
  connect(rightslider, SIGNAL(valueChanged(int)), right, SLOT(display(int)) ); 
  connect(rightslider, SIGNAL(valueChanged(int)), this, SLOT(setRight(int)) );
  
  QCheckBox *cd = new QCheckBox("Central Differencing", this);
  connect(cd, SIGNAL( toggled(bool) ), this, SLOT( toggleCDiff(bool) ));

  QCheckBox *ad = new QCheckBox("Derivative Averaging", this);
  connect(ad, SIGNAL( toggled(bool) ), this, SLOT( toggleDiffAvg(bool) ));  

  box1->addWidget(alggrp);
  box3->addWidget(threshgrp);
  box4->addWidget(hsgrp);
  vbox->addWidget(winlabel);
  vbox->addWidget(win);
  vbox->addWidget(winslider);
  vbox->addWidget(deglabel);
  vbox->addWidget(deg);
  vbox->addWidget(degslider);
  vbox->addWidget(leftlabel);
  vbox->addWidget(left);
  vbox->addWidget(leftslider);
  vbox->addWidget(rightlabel);
  vbox->addWidget(right);
  vbox->addWidget(rightslider);
  vbox->addWidget(cd);
  vbox->addWidget(ad);
  QHBoxLayout *box2 = new QHBoxLayout( vbox );
  box2->addWidget(_execbtn);
  box2->addWidget(_closebtn); 
  this->setCaption("Numerical Derivative Tools");
}

// destructor
NDiffToolDialog::~NDiffToolDialog()
{
   delete _ndifftools;
   delete _execbtn; 
   delete _closebtn;
}

void NDiffToolDialog::exec()
{
   // make sure we have a data set
   if (!_gbw->getDataPresent()) {
       _gbw->getStatusBar()->message("No data set loaded.", 5000); 
       return;
   }
   
    QApplication::setOverrideCursor(Qt::waitCursor);
   // switch on the analyzer type
   switch(_ntype) {
    case tNDiff:
      _ndifftools->prepare();
      _ndifftools->computeSimpleND();
      _ndifftools->displaySimpleND();
      _gbw->getStatusBar()->message("Numerical Derivatives Computed with No Smoothing.", 8000);
      break;
    case tLSOrthog:  
      _ndifftools->prepare();
      _ndifftools->computeNDLSOrtho(_win, _deg);
      _ndifftools->displayNDLSOrtho(_win);
      _gbw->getStatusBar()->message("Numerical Derivative via LS Ortho Computed.", 8000);
      break; 
    case tSavGolay: 
      _ndifftools->prepare();
      _ndifftools->computeNDSavGolay(_left, _right, _deg);
      _ndifftools->displayNDSavGolay(_left, _right, _deg); 
      _gbw->getStatusBar()->message("Numerical Derivative via Savitzky-Golay Computed.", 8000);
      break;
    case tWavedb4:
      _ndifftools->prepare();
      _ndifftools->computeNDWavelet(_left, _right, _deg,"'db4'");
      _ndifftools->displayNDWavelet(_left, _right, _deg,"'db4'");
      _gbw->getStatusBar()->message("Numerical Derivative via Wavelet DB4 Computed.", 8000);
      break;
    case tWavedb8:
      _ndifftools->prepare();
      _ndifftools->computeNDWavelet(_left, _right, _deg,"'db8'");
      _ndifftools->displayNDWavelet(_left, _right, _deg,"'db8'");
      _gbw->getStatusBar()->message("Numerical Derivative via Wavelet DB8 Computed.", 8000);
      break;
    case tWavesym8:
      _ndifftools->prepare();
      _ndifftools->computeNDWavelet(_left, _right, _deg,"'sym8'");
      _ndifftools->displayNDWavelet(_left, _right, _deg,"'sym8'");
      _gbw->getStatusBar()->message("Numerical Derivative via Wavelet SYM8 Computed.", 8000);
      break;
    case tMollify:
      _ndifftools->prepare();
      _ndifftools->computeNDMollify(_win);
      _ndifftools->displayNDMollify(_win);
      _gbw->getStatusBar()->message("Numerical Derivative via Mollification Computed.", 8000);
      break;
    case tABodd:
      _ndifftools->prepare();
      _ndifftools->computeNDABKP(1,_win);
      _ndifftools->displayNDABKP(1,_win);
      _gbw->getStatusBar()->message("Numerical Derivative via Anderssen-Bloomfield Regularization (Odd)", 8000);
      break;
    case tABnoodd:
      _ndifftools->prepare();
      _ndifftools->computeNDABKP(2,_win);
      _ndifftools->displayNDABKP(2,_win);
      _gbw->getStatusBar()->message("Numerical Derivative via Anderssen-Bloomfield Regularization (No Odd)", 8000);
      break; 
    case tKPodd:
      _ndifftools->prepare();
      _ndifftools->computeNDABKP(3,_win);
      _ndifftools->displayNDABKP(3,_win);
      _gbw->getStatusBar()->message("Numerical Derivative via Kosarev-Pantos Window (Odd)", 8000);
      break;    
    case tKPnoodd:
      _ndifftools->prepare();
      _ndifftools->computeNDABKP(4,_win);
      _ndifftools->displayNDABKP(4,_win);
      _gbw->getStatusBar()->message("Numerical Derivative via Kosarev-Pantos Window (No Odd)", 8000);
      break;    
     default:
       _gbw->getStatusBar()->message("No Analysis Tool Selected.", 8000);
   }
    QApplication::restoreOverrideCursor();
}

